Struct PrimitiveBlock

pub struct PrimitiveBlock {
    pub stringtable: StringTable,
    pub primitivegroup: Vec<PrimitiveGroup>,
    pub granularity: Option<i32>,
    pub lat_offset: Option<i64>,
    pub lon_offset: Option<i64>,
    pub date_granularity: Option<i32>,
}

Fields

stringtable: StringTableprimitivegroup: Vec<PrimitiveGroup>granularity: Option<i32>

Granularity, units of nanodegrees, used to store coordinates in this block.

lat_offset: Option<i64>

Offset value between the output coordinates and the granularity grid in units of nanodegrees.

lon_offset: Option<i64>date_granularity: Option<i32>

Granularity of dates, normally represented in units of milliseconds since the 1970 epoch.

Implementations

Source

impl PrimitiveBlock

pub fn granularity(&self) -> i32

Returns the value of granularity, or the default value if granularity is unset.

pub fn date_granularity(&self) -> i32

Returns the value of date_granularity, or the default value if date_granularity is unset.

pub fn lat_offset(&self) -> i64

Returns the value of lat_offset, or the default value if lat_offset is unset.

pub fn lon_offset(&self) -> i64

Returns the value of lon_offset, or the default value if lon_offset is unset.