pub struct Way {
pub id: i64,
pub keys: Vec<u32>,
pub vals: Vec<u32>,
pub info: Option<Info>,
pub refs: Vec<i64>,
pub lat: Vec<i64>,
pub lon: Vec<i64>,
}Fields
id: i64keys: Vec<u32>Parallel arrays.
vals: Vec<u32>info: Option<Info>refs: Vec<i64>DELTA coded
lat: Vec<i64>The following two fields are optional. They are only used in a special format where node locations are also added to the ways. This makes the files larger, but allows creating way geometries directly.
If this is used, you MUST set the optional_features tag “LocationsOnWays” and the number of values in refs, lat, and lon MUST be the same.
DELTA coded, optional
lon: Vec<i64>DELTA coded, optional